-- Event Timer by David Down

import "Turbine.Gameplay"
import "Turbine.UI.Lotro"

function print(text) Turbine.Shell.WriteLine("<rgb=#00FFFF>ET:</rgb> "..text) end

local function cnt(list)
	local n = 0
	for k in pairs(list) do n=n+1; end
	return n
end

Event = Turbine.PluginData.Load( Turbine.DataScope.Server,"ET_Data")
if type(Event) ~= "table" then
	Event = {[0]=false}
	print("Event Timer data initialized.")
else print("Event Timer data loaded ("..cnt(Event)..").")
end

-- Save player name for event entries
player = Turbine.Gameplay.LocalPlayer.GetInstance()
pname = player:GetName()

local function GetTime(str)
    local hr = tonumber(str)
    if hr and hr>0 and hr<170 then
        local now = Turbine.Engine.GetLocalTime()
        local time = now + 3600*hr
        return time
    end
    print("<rgb=#FF0000>Error, hours must be < 170</rgb>" )
end

ET_Command = Turbine.ShellCommand()

function ET_Command:Execute( command,args )
    if args=="list" or args=="" then
		Turbine.Shell.WriteLine( "Event Timer events:" )
		ix = {}
		local nr,now,hr = 0, Turbine.Engine.GetLocalTime()
		for time,desc in pairs(Event) do
			if time>0 then
				hr = (time-now)/3600
				nr = nr+1
				ix[nr] = time
			end
		end
		if nr<1 then
            print("No active events" )
            return
        end
		table.sort(ix)
		Turbine.Shell.WriteLine( " # Hours Description" )
		for nr,time in ipairs(ix) do
			hr = (time-now)/3600
		    Turbine.Shell.WriteLine( string.format("%2i %6.1f %s",nr,hr,Event[time]) )
        end
    elseif args=="clear" then
		Event = {[-1]=Event[-1], [0]=Event[0]}
        print("All events CLEARED")
    elseif args=="name" then
        Event[0] = not Event[0]
        print("Name append: "..tostring(Event[0]))
    elseif args=="now" then
        local now = Turbine.Engine.GetLocalTime()
        print("now = "..now)
    else
        if args:match("^%d+$") then
            local n = tonumber(args)
            if ix and ix[n] then
                print("Removing "..Event[ix[n]] )
                Event[ix[n]] = nil
            else print("<rgb=#FF0000>Error, no such event #</rgb> ("..n..")") end
            return
        end
        local line,hours = args:match("^(%d+) (%d+%.?%d?)$")
        if line then
            local n = tonumber(line)
            local time = GetTime(hours)
            if ix and ix[n] and time then
                Event[time] = Event[ix[n]]
                Event[ix[n]] = nil
                ET_Command:Execute( nil,"" ) -- Redisplay list
            else print("<rgb=#FF0000>Error, no such event #</rgb> ("..n..")") end
            return
        end
        local hours,desc = args:match("^(%d+%.?%d?) (.+)$")
        if hours then
            local time = GetTime(hours)
            if not time then return end
			if Event[0] then desc = desc.." ("..pname..")" end
            Event[time] = desc
            ET_Command:Execute( nil,"" ) -- Redisplay list
        else Turbine.Shell.WriteLine( "?? "..self:GetShortHelp() ); end
    end
end

function ET_Command:GetHelp()
	return [[Event Timer commands:
et: Display event list
et clear: Clear all events
et name: Toggle name appending
et #: Delete event on line #
et # <hrs>: Change event time on line #
et # <desc>: Add event in # hours]]
end

function ET_Command:GetShortHelp()
	return "Event Timer ('/help et' for command list)"
end

Turbine.Shell.AddCommand( "et", ET_Command )

Plugins.EventTimer.Open = function(sender,args)
	ET_Command:Execute("et","")
end

Plugins.EventTimer.Unload = function(sender,args)
    Turbine.PluginData.Save(Turbine.DataScope.Server,"ET_Data",Event)
    print("Event Timer data saved.")
end

if Event[-1] then ET_Command:Execute("et","") end

-- Options panel
OP = Turbine.UI.Control()
OP:SetBackColor( Turbine.UI.Color(0.0, 0.0, 0.1) )
OP:SetSize( 240, 260 )

local autoBox = Turbine.UI.Lotro.CheckBox()
autoBox:SetText( " Display list on load" )
autoBox:SetSize( 170, 22 )
autoBox:SetPosition( 10, 10 )
autoBox:SetParent( OP )
if Event[-1] then autoBox:SetChecked(true) end

autoBox.CheckedChanged = function( sender, args )
	Event[-1] = sender:IsChecked()
	print((Event[-1] and "En" or "Dis").."abled Auto-display.")
end

plugin.GetOptionsPanel = function( self ) return OP end
